!--------+---------+---------+---------+---------+---------+---------+---------|
      subroutine KMATH_EIGEN_GEV( n, a, lda, b, ldb, w, z, ldz,
     &                            mode, solver, info )
!--------+---------+---------+---------+---------+---------+---------+---------|
!
!   integer(In)    n        :: Matrix dimension
!   real(8)(InOut) a(lda,*) :: An input matrix to be solved for GEV of LHS.
!                              A is real symmetric matrix,
!                              and an upper triangular format distributed on
!                              a 2D cyclic-cyclic fashion.
!                              On exit, data is destroyed.
!   integer(In)    lda      :: Leading Dimension of the array A
!   real(8)(InOut) b(ldb,*) :: An input matrix to be solved for GEV of RHS.
!                              B is real positive definite symmetric matrix,
!                              and an upper triangular format distributed on
!                              a 2D cyclic-cyclic fashion.
!                              The transformation factor to SEP,
!                                 eig.vec(B) * diag(eig.val(B))^{-1/2}.
!                              is returned on exit.
!   integer(In)    ldb      :: Leading Dimension of the array B
!   real(8)(Out)   w(*)     :: An array where eigenvalues computed are stored
!   real(8)(Out)   z(ldz,*) :: An output matrix where eigenvectors are stored
!   integer(In)    ldz      :: Leading Dimension of the array Z
!   character*1(In)  mode   :: Computing mode
!   character*1(In)  solver :: Solver
!   integer(Out)   info     :: status information
!
      use MPI
!$    use OMP_LIB
      use eigen_libs
      implicit none

      integer, intent(In)    ::  n, lda, ldb, ldz
      real(8), intent(InOut) ::  a(*), b(*)
      real(8), intent(InOut) ::  w(*), z(*)
      character*1, intent(In),  optional ::  mode, solver
      integer,     intent(Out), optional ::  info

      integer :: nprocs, nprow, npcol, ierr
      integer :: my_id, x_id, y_id
      integer :: version
      integer, parameter :: version_required = 234 ! 2.3d
      logical :: flag
      character*1 :: mode_, solver_
      integer :: info_

      info_ = 0

      call eigen_initialized( flag )
      if ( .NOT. flag ) then
         print*,"***********"
         print*,"** ERROR **"
         print*,"***********"
         print*,"EigenExa was not initialized, and terminated."
         call MPI_Abort( MPI_COMM_WORLD, 1, ierr )
      end if

      call eigen_get_procs( nprocs, nprow, npcol )
      call eigen_get_id( my_id, x_id, y_id )

      call eigen_get_version( version )
      if ( version < version_required ) then
      if ( my_id == 1 ) then
         print*,"*************"
         print*,"** CAUTION **"
         print*,"*************"
         print*,"The version of EigenExa is older than the one"
         print*,"this GEV driver requires."
      end if
      end if

      mode_ = 'A'
      if ( PRESENT(mode) ) then
         if ( mode == 'A' .OR. mode == 'X'
     &                    .OR. mode == 'N' ) then
            mode_ = mode
         end if
      end if

      solver_ = 'S'
      if ( PRESENT(solver) ) then
         if ( solver == 'S' .OR. solver == 'X' ) then
            solver_ = solver
         end if
      end if

      if ( nprow == npcol ) then

         call KMATH_EIGEN_GEV_1( n, a, lda, b, ldb, w, z, ldz,
     &                           mode_, solver_, info_ )

      else

         call KMATH_EIGEN_GEV_2( n, a, lda, b, ldb, w, z, ldz,
     &                           mode_, solver_, info_ )

      end if

      if ( PRESENT(info) ) then
         info = info_
      end if

      return
!--------+---------+---------+---------+---------+---------+---------+---------|
      contains

      include 'KMATH_EIGEN_GEV_1.F'

      include 'KMATH_EIGEN_GEV_2.F'

      include 'KMATH_EIGEN_GEV_misc.F'

!--------+---------+---------+---------+---------+---------+---------+---------|
      end subroutine KMATH_EIGEN_GEV
!--------+---------+---------+---------+---------+---------+---------+---------|

